# -*- coding: utf-8 -*-

import os
import mflog

UNIT_TESTS_MODE = (os.environ.get('_MFLOG_UNITTESTS', '0') == '1')
UNIT_TESTS_STDOUT = []
UNIT_TESTS_STDERR = []
UNIT_TESTS_JSON = []


def reset_unittests():
    """Reset unittests."""
    global UNIT_TESTS_STDOUT, UNIT_TESTS_STDERR, UNIT_TESTS_JSON
    del(UNIT_TESTS_STDOUT[:])
    del(UNIT_TESTS_STDERR[:])
    del(UNIT_TESTS_JSON[:])
    try:
        del(os.environ["MFLOG_EXTRA_CONTEXT_FUNC"])
    except Exception:
        pass
    try:
        del(os.environ["MFLOG_JSON_ONLY_KEYS"])
    except Exception:
        pass
    mflog.__unset_configuration()


def extra_context():
    return {"extra_context_key1": "extra_context_value1",
            "extra_context_key2": "extra_context_value2"}



from logging.handlers import SysLogHandler
from logging import LogRecord
import json


class SyslogLoggerMsgOnlyFormatter(object):

    def format(self, record):
        return record.msg['event']


class SyslogLoggerJSONFormatter(object):

    def format(self, record):
        return json.dumps(record.msg)


class SyslogLogger(object):

    __syslog_handler = None

    def __init__(self, address, frmt=None):
        self.__syslog_handler = SysLogHandler(address)
        if frmt is None or frmt == "msg_only":
            self.__syslog_handler.formatter = SyslogLoggerMsgOnlyFormatter()
        else:
            self.__syslog_handler.formatter = SyslogLoggerJSONFormatter()

    def close(self):
        self.__syslog_handler.close()

    def msg(self, event_dict):
        record = LogRecord(event_dict.get("name", "unknown"),
                           event_dict.get("level", "WARNING"),
                           "/not_used/not_used.py", 1,
                           event_dict, [], None)
        self.__syslog_handler.acquire()
        try:
            self.__syslog_handler.emit(record)
        finally:
            self.__syslog_handler.release()



# -*- coding: utf-8 -*-

from __future__ import print_function
import os
import fnmatch
import logging
import fcntl
import sys
import six
import importlib
import inspect
try:
    from rich.console import Console
    from rich.tabulate import tabulate_mapping
except ImportError:
    pass

OVERRIDE_LINES_CACHE = None
LEVEL_FROM_LOGGER_NAME_CACHE = {}


def write_with_lock(f, message):
    fcntl.flock(f, fcntl.LOCK_EX)
    f.write(message)


def flush_with_lock(f):
    f.flush()
    fcntl.flock(f, fcntl.LOCK_UN)


def __reset_level_from_logger_name_cache():
    global LEVEL_FROM_LOGGER_NAME_CACHE
    LEVEL_FROM_LOGGER_NAME_CACHE = {}


def get_func_by_path(func_path):
    func_name = func_path.split('.')[-1]
    module_path = ".".join(func_path.split('.')[0:-1])
    if module_path == "":
        print("ERROR: %s must follow 'pkg.function_name'" % func_path,
              file=sys.stderr)
        sys.exit(1)
    if func_path.endswith(')'):
        print("ERROR: %s must follow 'pkg.function_name'" % func_path,
              file=sys.stderr)
        print("=> EXIT", file=sys.stderr)
        sys.exit(1)
    try:
        mod = importlib.import_module(module_path)
    except Exception:
        print("ERROR: can't import %s" % module_path, file=sys.stderr)
        print("=> EXIT", file=sys.stderr)
        sys.exit(1)
    try:
        return getattr(mod, func_name)
    except Exception:
        print("ERROR: can't get %s on %s" % (func_name, module_path),
              file=sys.stderr)
        print("=> EXIT", file=sys.stderr)
        sys.exit(1)


class classproperty(object):

    def __init__(self, f):
        self.f = f

    def __get__(self, obj, owner):
        return self.f(owner)


class Config(object):

    _instance = None
    _minimal_level = None
    _json_minimal_level = None
    _json_file = None
    _override_files = None
    _override_dict = None
    _extra_context_func = None
    _json_only_keys = None
    _syslog_address = None
    _syslog_format = None
    _syslog_minimal_level = None
    _fancy_output = None
    _auto_dump_locals = True

    def __init__(self, minimal_level=None, json_minimal_level=None,
                 json_file=None, override_files=None,
                 thread_local_context=False,
                 extra_context_func=None, json_only_keys=None,
                 override_dict={}, syslog_address=None, syslog_format=None,
                 syslog_minimal_level=None,
                 fancy_output=None,
                 auto_dump_locals=True):
        global LEVEL_FROM_LOGGER_NAME_CACHE, OVERRIDE_LINES_CACHE
        OVERRIDE_LINES_CACHE = {}
        LEVEL_FROM_LOGGER_NAME_CACHE = {}
        if minimal_level is not None:
            self._minimal_level = minimal_level
        else:
            self._minimal_level = os.environ.get('MFLOG_MINIMAL_LEVEL', 'INFO')
        if json_minimal_level is not None:
            self._json_minimal_level = json_minimal_level
        else:
            self._json_minimal_level = \
                os.environ.get('MFLOG_JSON_MINIMAL_LEVEL', 'WARNING')
        if syslog_minimal_level is not None:
            self._syslog_minimal_level = syslog_minimal_level
        else:
            self._syslog_minimal_level = \
                os.environ.get('MFLOG_SYSLOG_MINIMAL_LEVEL', 'WARNING')
        if syslog_format is not None:
            self._syslog_format = syslog_format
        else:
            self._syslog_format = \
                os.environ.get('MFLOG_SYSLOG_FORMAT', 'null')
        if self._syslog_format not in ('null', 'msg_only', 'json'):
            raise Exception("unknown syslog format: %s => must be null, "
                            "msg_only or json")
        if self._syslog_format == "null":
            self._syslog_format = None
        if json_file is not None:
            self._json_file = json_file
        else:
            self._json_file = os.environ.get("MFLOG_JSON_FILE", None)
            if self._json_file == "null":
                self._json_file = None
        if syslog_address is not None:
            tmpsyslog = syslog_address
        else:
            tmpsyslog = os.environ.get("MFLOG_SYSLOG_ADDRESS", None)
            if tmpsyslog == "null":
                tmpsyslog = None
        if isinstance(tmpsyslog, six.string_types):
            tmpsyslog2 = tmpsyslog.split(':')
            if len(tmpsyslog2) == 1:
                self._syslog_address = (tmpsyslog2[0], 514)
            elif len(tmpsyslog2) == 2:
                self._syslog_address = (tmpsyslog2[0], int(tmpsyslog2[1]))
            else:
                raise Exception("wrong syslog_address type: %s" % tmpsyslog)
        if override_files is not None:
            self._override_files = override_files
        else:
            self._override_files = \
                [x.strip() for x in os.environ.get(
                    "MFLOG_MINIMAL_LEVEL_OVERRIDE_FILES", "").split(';')]
        if extra_context_func is not None:
            self._extra_context_func = extra_context_func
        else:
            if "MFLOG_EXTRA_CONTEXT_FUNC" in os.environ:
                self._extra_context_func = \
                    get_func_by_path(os.environ['MFLOG_EXTRA_CONTEXT_FUNC'])
            else:
                self._extra_context_func = None
        if self._extra_context_func and not callable(self._extra_context_func):
            print("ERROR: extra_context_func must be a python callable",
                  file=sys.stderr)
            print("=> EXIT", file=sys.stderr)
            sys.exit(1)
        if json_only_keys is not None:
            self._json_only_keys = json_only_keys
        else:
            if "MFLOG_JSON_ONLY_KEYS" in os.environ:
                self._json_only_keys = \
                    os.environ["MFLOG_JSON_ONLY_KEYS"].split(',')
            else:
                self._json_only_keys = []
        self._override_dict = override_dict
        if fancy_output is None:
            if 'rich' in sys.modules:
                self._fancy_output = None
            else:
                self._fancy_output = False
        else:
            self._fancy_output = fancy_output
        self._auto_dump_locals = auto_dump_locals

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = Config()
        return cls._instance

    @classmethod
    def set_instance(cls, *args, **kwargs):
        cls._instance = Config(*args, **kwargs)

    @classproperty
    def minimal_level(cls):  # pylint: disable=E0213
        return cls.get_instance()._minimal_level

    @classproperty
    def auto_dump_locals(cls):  # pylint: disable=E0213
        return cls.get_instance()._auto_dump_locals

    @classproperty
    def extra_context_func(cls):  # pylint: disable=E0213
        return cls.get_instance()._extra_context_func

    @classproperty
    def json_minimal_level(cls):  # pylint: disable=E0213
        return cls.get_instance()._json_minimal_level

    @classproperty
    def json_file(cls):  # pylint: disable=E0213
        return cls.get_instance()._json_file

    @classproperty
    def override_files(cls):  # pylint: disable=E0213
        return cls.get_instance()._override_files

    @classproperty
    def syslog_minimal_level(cls):  # pylint: disable=E0213
        return cls.get_instance()._syslog_minimal_level

    @classproperty
    def syslog_address(cls):  # pylint: disable=E0213
        return cls.get_instance()._syslog_address

    @classproperty
    def syslog_format(cls):  # pylint: disable=E0213
        return cls.get_instance()._syslog_format

    @classproperty
    def override_dict(cls):  # pylint: disable=E0213
        return cls.get_instance()._override_dict

    @classproperty
    def json_only_keys(cls):  # pylint: disable=E0213
        return cls.get_instance()._json_only_keys

    @classproperty
    def fancy_output(cls):  # pylint: disable=E0213
        return cls.get_instance()._fancy_output


def level_name_to_level_no(level_name):
    """Convert level_name (debug, WARNING...) to level number.

    Args:
        level_name (string): A level name (debug, warning, info, critical
            or errot), case insensitive.

    Returns:
        (int) Corresponding level number (in logging library).

    Raises:
        Exception: if the level in unknown.

    """
    ulevel_name = level_name.upper()
    if ulevel_name == "DEBUG" or ulevel_name == "NOTSET":
        return logging.DEBUG
    elif ulevel_name == "INFO":
        return logging.INFO
    elif ulevel_name == "WARNING":
        return logging.WARNING
    elif ulevel_name == "ERROR" or ulevel_name == "EXCEPTION":
        return logging.ERROR
    elif ulevel_name == "CRITICAL":
        return logging.CRITICAL
    else:
        raise Exception("unknown level name: %s" % level_name)


def _file_to_lines(file_path):
    """Read the given file_path and decode mflog_override format.

    foo.bar.* => DEBUG
    foo.* => WARNING

    Notes:
    - lines beginning with # are ignored
    - the left part is a read as a fnmatch pattern
    - the right part is a case insensitive level name

    Args:
        file_path (string): The full path of the file to read.

    Returns:
        (list of couples) A list of (logger name pattern, level name).

    """
    try:
        with open(file_path, "r") as f:
            tmp = [x.split('=>') for x in f.readlines()]
            lines = [(x[0].strip(), x[1].strip()) for x in tmp
                     if len(x) == 2 and not x[0].strip().startswith('#')]
        return lines
    except IOError:
        return []


def _get_override_lines(path):
    """Read the custom level override configuration file (if exists).

    Args:
        path (string): the full path of the override configuration file.

    Note: the content is cached (in memory).

    Returns:
        (list of couples) A list of (logger name pattern, level name).

    """
    global OVERRIDE_LINES_CACHE
    if path not in OVERRIDE_LINES_CACHE:
        lines = _file_to_lines(path)
        OVERRIDE_LINES_CACHE[path] = lines
    return OVERRIDE_LINES_CACHE[path]


def get_extra_context():
    """Return an extra context by calling an external configured python func.

    Returns:
        (dict) A dict of extra context key/values as strings.

    """
    extra_context_f = Config.extra_context_func
    if extra_context_f is None:
        return {}
    extra_context = extra_context_f()  # pylint: disable=E1120
    if not isinstance(extra_context, dict):
        print("bad extra_context (not a dict) => ignoring", file=sys.stderr)
        return {}
    return extra_context


def get_level_no_from_logger_name(logger_name):
    """Get the level number to use for the given logger name.

    Note:
      - first we search in override_dict, if there is a match, it's over
      - (if no match) at first step, we check each files in override_files
        configuration. The first match wins.
      - if there is no match, we return the default level number.

    Note: the result is cached in memory.

    Args:
        logger_name (string): The logger name.

    Returns:
        (int) The level number to use for this logger name.

    """
    global LEVEL_FROM_LOGGER_NAME_CACHE

    class Found(Exception):
        pass

    if logger_name not in LEVEL_FROM_LOGGER_NAME_CACHE:
        # pylint: disable=no-member
        for k, v in Config.override_dict.items():
            if fnmatch.fnmatch(logger_name, k):
                LEVEL_FROM_LOGGER_NAME_CACHE[logger_name] = \
                    level_name_to_level_no(v)
                return LEVEL_FROM_LOGGER_NAME_CACHE[logger_name]
        paths = Config.override_files
        try:
            for path in paths:  # pylint: disable=E1133
                custom_lines = _get_override_lines(path)
                for k, v in custom_lines:
                    if fnmatch.fnmatch(logger_name, k):
                        LEVEL_FROM_LOGGER_NAME_CACHE[logger_name] = \
                            level_name_to_level_no(v)
                        raise Found
        except Found:
            pass
        else:
            LEVEL_FROM_LOGGER_NAME_CACHE[logger_name] = \
                level_name_to_level_no(Config.minimal_level)
    return LEVEL_FROM_LOGGER_NAME_CACHE[logger_name]


def get_resolved_fancy_output_config_value(f=sys.stderr):
    fancy = Config.fancy_output
    if fancy is None:
        try:
            fancy = f.isatty()
        except Exception:
            fancy = False
    return fancy


def dump_locals(f=sys.stderr):
    fancy = get_resolved_fancy_output_config_value(f=f)
    stack_offset = -1
    try:
        caller = inspect.stack()[stack_offset]
        locals_map = {
            key: value
            for key, value in caller.frame.f_locals.items()
            if not key.startswith("__")
        }
        for k, v in locals_map.items():
            if len(repr(v)) > 10000:
                locals_map[k] = \
                    "(too big value => hidden in this variables dump)"
        if fancy:
            c = Console(file=sys.stderr)
            c.print(tabulate_mapping(locals_map, title="Locals"))
        else:
            print("Locals dump", file=f)
            for k, v in locals_map.items():
                print("%s: %r" % (k, v))
    except Exception:
        return False
    return True



# -*- coding: utf-8 -*-

import os
import structlog
from mflog.utils import level_name_to_level_no, get_level_no_from_logger_name
from mflog.utils import get_extra_context


def fltr(logger, method_name, event_dict):
    """Filter log messages."""
    method_level_no = level_name_to_level_no(method_name)
    logger_level_no = \
        get_level_no_from_logger_name(event_dict.get('name', ''))
    if method_level_no < logger_level_no:
        raise structlog.DropEvent
    return event_dict


def add_level(logger, method_name, event_dict):
    """Add the severity in the event_dict."""
    event_dict['level'] = method_name.lower()
    return event_dict


def add_pid(logger, method_name, event_dict):
    """Add the current pid in the event dict."""
    event_dict['pid'] = os.getpid()
    return event_dict


def add_extra_context(logger, method_name, event_dict):
    """Add extra context in the event dict."""
    extra_context = get_extra_context()
    event_dict.update(extra_context)
    return event_dict


def add_exception_info(logger, method_name, event_dict):
    exc_info = event_dict.pop("exc_info", None)
    if exc_info:
        e = structlog.processors._figure_out_exc_info(exc_info)
        if e[0] is not None:
            event_dict["exception"] = structlog._frames._format_exception(e)
            event_dict["exception_type"] = e[0].__name__
            event_dict["exception_file"] = e[-1].tb_frame.f_code.co_filename
            event = event_dict.get("event", "")
            if isinstance(event, Exception):
                # see issue #3
                # => we convert the exception object in string to avoid
                #    json serializing issues
                event_dict["event"] = str(event)
        else:
            event_dict["exception"] = None
        return event_dict
    return event_dict


def kv_renderer(logger, method_name, event_dict):
    ordered_items = sorted(event_dict.items())
    return " ".join(["%s=%s" % (k, v) for k, v in ordered_items])



# -*- coding: utf-8 -*-

from __future__ import print_function
import sys
import json
import os
import logging
import logging.config
import structlog
import functools
import traceback
try:
    from rich.console import Console
    from rich.table import Table
    from rich.text import Text
except ImportError:
    pass

from mflog.utils import level_name_to_level_no, Config, \
    get_level_no_from_logger_name, write_with_lock, flush_with_lock, \
    __reset_level_from_logger_name_cache, \
    get_resolved_fancy_output_config_value
from mflog.utils import dump_locals as _dump_locals
from mflog.processors import fltr, add_level, add_pid, add_exception_info, \
    kv_renderer, add_extra_context
from mflog.unittests import UNIT_TESTS_STDOUT, UNIT_TESTS_STDERR, \
    UNIT_TESTS_JSON, UNIT_TESTS_MODE
from mflog.syslog import SyslogLogger

CONFIGURATION_SET = False


class StructlogHandler(logging.Handler):
    """Feed all events back into `structlog`.

    See https://github.com/hynek/structlog/issues/9
    """

    level = logging.DEBUG
    filters = []
    lock = None
    __loggers = None

    def __init__(self, *args, **kwargs):
        logging.Handler.__init__(self, *args, **kwargs)
        self.__loggers = {}

    def __get_logger(self, name):
        if name not in self.__loggers:
            self.__loggers[name] = get_logger(name)
        return self.__loggers[name]

    def emit(self, record):
        kwargs = {'name': record.name}
        if record.exc_info:
            kwargs['exc_info'] = record.exc_info
        logger = self.__get_logger(record.name)
        if record.levelno >= logging.CRITICAL:
            f = logger.critical
        elif record.levelno >= logging.ERROR:
            f = logger.error
        elif record.levelno >= logging.WARNING:
            f = logger.warning
        elif record.levelno >= logging.INFO:
            f = logger.info
        elif record.levelno >= logging.DEBUG:
            f = logger.debug
        else:
            # let's ignore this
            return
        # Mimick the formatting behaviour of the stdlib's logging
        # module, which accepts both positional arguments and a single
        # dict argument.
        if record.args and isinstance(record.args, dict):
            f(record.msg, record.args, **kwargs)
        else:
            f(record.msg, *(record.args), **kwargs)


class MFLogLogger(object):

    _unittests_stdout = None
    _unittests_stderr = None
    _unittests_json = None

    def __init__(self, *args):
        self._json_file = None
        self._json_logger = None
        self._syslog_logger = None
        if len(args) > 0:
            self.name = args[0]
        else:
            self.name = 'root'
        self._stdout_print_logger = structlog.PrintLogger(sys.stdout)
        self._stderr_print_logger = structlog.PrintLogger(sys.stderr)
        if Config.syslog_address:
            self._syslog_logger = SyslogLogger(Config.syslog_address,
                                               Config.syslog_format)
        if Config.json_file or UNIT_TESTS_MODE:
            if UNIT_TESTS_MODE or Config.json_file is None:
                self._json_file = open('/dev/null', 'a')
                self._json_logger = structlog.PrintLogger(self._json_file)
            else:
                self._json_file = open(Config.json_file, 'a')
                self._json_logger = structlog.PrintLogger(self._json_file)
                self._json_logger._write = functools.partial(write_with_lock,
                                                             self._json_file)
                self._json_logger._flush = functools.partial(flush_with_lock,
                                                             self._json_file)
        if UNIT_TESTS_MODE:
            self._stdout_print_logger._flush = lambda *args, **kwargs: None
            self._stdout_print_logger._write = UNIT_TESTS_STDOUT.append
            self._stderr_print_logger._flush = lambda *args, **kwargs: None
            self._stderr_print_logger._write = UNIT_TESTS_STDERR.append
            self._json_logger._flush = lambda *args, **kwargs: None
            self._json_logger._write = UNIT_TESTS_JSON.append
        self._json_only_keys = Config.json_only_keys

    def close(self):
        if self._json_file:
            try:
                self._json_file.close()
            except Exception:
                pass
        if self._syslog_logger is not None:
            self._syslog_logger.close()

    def __del__(self):
        self.close()

    def _msg(self, std_logger, **event_dict):
        try:
            self._json(**event_dict)
        except Exception as e:
            print("MFLOG ERROR: can't write log message to json output "
                  "with exception: %s" % e, file=sys.stderr)
            traceback.print_exc(file=sys.stderr)
        try:
            self._syslog(**event_dict)
        except Exception as e:
            print("MFLOG ERROR: can't write log message to syslog output "
                  "with exception: %s" % e, file=sys.stderr)
            traceback.print_exc(file=sys.stderr)
        fancy = get_resolved_fancy_output_config_value(f=std_logger._file)
        if fancy:
            try:
                self._fancy_msg(std_logger._file, **event_dict)
            except Exception as e:
                print("MFLOG ERROR: can't write log message to fancy output "
                      "with exception: %s" % e, file=sys.stderr)
                traceback.print_exc(file=sys.stderr)
        else:
            try:
                std_logger.msg(self._format(event_dict))
            except Exception as e:
                print("MFLOG ERROR: can't write log message to stdout/err "
                      "with exception: %s" % e, file=sys.stderr)
                traceback.print_exc(file=sys.stderr)

    def _fancy_msg(self, f, **event_dict):
        c = Console(file=f, highlight=False, emoji=False, markup=False)
        lll = event_dict.pop('level').lower()
        llu = lll.upper()
        exc = event_dict.pop('exception', None)
        event_dict.pop('exception_type', None)
        event_dict.pop('exception_file', None)
        name = event_dict.pop('name', 'root')
        pid = event_dict.pop('pid')
        ts = event_dict.pop('timestamp')[0:-3] + "Z"
        try:
            msg = event_dict.pop('event')
        except KeyError:
            msg = "None"
        for key in self._json_only_keys:  # pylint: disable=E1133
            try:
                event_dict.pop(key)
            except KeyError:
                pass
        extra = ""
        if len(event_dict) > 0:
            extra = kv_renderer(None, None, event_dict)
        if lll in ['notset', 'debug', 'info', 'warning',
                   'error', 'critical']:
            ls = "logging.level.%s" % lll
        else:
            ls = "none"
        output = Table(show_header=False, expand=True, box=None,
                       padding=(0, 1, 0, 0))
        output.add_column(style="log.time")
        output.add_column(width=10, justify="center")
        output.add_column(justify="center")
        output.add_column(ratio=1)
        row = []
        row.append(Text(ts))
        row.append(Text("[%s]" % llu, style=ls))
        row.append(Text(name, style="bold") + Text("#") + Text("%i" % pid,
                                                               style="yellow"))
        row.append(Text(msg))
        output.add_row(*row)
        if extra != "":
            output.add_row(
                "", "", "",
                Text("{ ", style="repr.attrib_name") +
                Text(extra, style="repr.attrib_name") +
                Text(" }", style="repr.attrib_name"))
        c.print(output)
        if exc is not None:
            c.print_exception()
            if Config.auto_dump_locals:
                _dump_locals(f)

    def _msg_stdout(self, **event_dict):
        self._msg(self._stdout_print_logger, **event_dict)

    def _msg_stderr(self, **event_dict):
        self._msg(self._stderr_print_logger, **event_dict)

    def _json(self, **event_dict):
        if Config.json_file is None and not UNIT_TESTS_MODE:
            return
        method_level_no = level_name_to_level_no(event_dict['level'])
        if method_level_no < level_name_to_level_no(Config.json_minimal_level):
            return
        self._json_logger.msg(json.dumps(event_dict))

    def _syslog(self, **event_dict):
        if Config.syslog_address is None:
            return
        method_level_no = level_name_to_level_no(event_dict['level'])
        syslog_minimal_level = Config.syslog_minimal_level
        if method_level_no < level_name_to_level_no(syslog_minimal_level):
            return
        self._syslog_logger.msg(event_dict)

    def _format(self, event_dict):
        level = "[%s]" % event_dict.pop('level').upper()
        ts = event_dict.pop('timestamp')
        name = event_dict.pop('name', 'root')
        pid = event_dict.pop('pid')
        try:
            msg = event_dict.pop('event')
        except KeyError:
            msg = "None"
        exc = event_dict.pop('exception', None)
        event_dict.pop('exception_type', None)
        event_dict.pop('exception_file', None)
        for key in self._json_only_keys:  # pylint: disable=E1133
            try:
                event_dict.pop(key)
            except KeyError:
                pass
        extra = ""
        if len(event_dict) > 0:
            extra = " {%s}" % kv_renderer(None, None, event_dict)
        tmp = "%s %10s (%s#%i) %s%s" % (ts, level, name, pid, msg, extra)
        if exc is not None:
            tmp = tmp + "\n" + exc
        return tmp

    def _json_format(self, event_dict):
        return json.dumps(event_dict)

    def isEnabledFor(self, level):
        logger_level_no = \
            get_level_no_from_logger_name(self.name)
        return level >= logger_level_no

    def getEffectiveLevel(self):
        return get_level_no_from_logger_name(self.name)

    def setLevel(self, level):
        pass

    debug = info = msg = _msg_stdout
    error = critical = warning = exception = _msg_stderr


class MFLogLoggerFactory(object):

    def __call__(self, *args):
        return MFLogLogger(*args)


class MFBoundLogger(structlog.stdlib.BoundLogger):

    def die(self, *args, **kwargs):
        if len(args) == 0:
            self.exception("die() called", **kwargs)
        else:
            self.exception(*args, **kwargs)
        if Config.auto_dump_locals:
            _dump_locals()
        sys.exit(1)

    def dump_locals(self):
        res = _dump_locals()
        if not res:
            self.warning("can't dump locals")


def set_config(minimal_level=None, json_minimal_level=None,
               json_file=None, override_files=None,
               thread_local_context=False, extra_context_func=None,
               json_only_keys=None, standard_logging_redirect=None,
               override_dict={}, syslog_address=None, syslog_format=None,
               fancy_output=None, auto_dump_locals=True):
    """Set the logging configuration.

    The configuration is cached. So you can call this several times.

    """
    global CONFIGURATION_SET
    Config.set_instance(minimal_level=minimal_level,
                        json_minimal_level=json_minimal_level,
                        json_file=json_file,
                        override_files=override_files,
                        thread_local_context=thread_local_context,
                        extra_context_func=extra_context_func,
                        json_only_keys=json_only_keys,
                        override_dict=override_dict,
                        syslog_address=syslog_address,
                        syslog_format=syslog_format,
                        fancy_output=fancy_output,
                        auto_dump_locals=auto_dump_locals)
    if standard_logging_redirect is not None:
        slr = standard_logging_redirect
    else:
        if 'MFLOG_STANDARD_LOGGING_REDIRECT' in os.environ:
            slr = (os.environ['MFLOG_STANDARD_LOGGING_REDIRECT'] == '1')
        else:
            slr = True  # default value
    if slr:
        # Configure standard logging redirect to structlog
        d = {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {},
            "handlers": {},
            "filters": {},
            "loggers": {
                "": {
                    "level": "NOTSET"
                }
            }
        }
        logging.config.dictConfig(d)
        root_logger = logging.getLogger()
        root_logger.addHandler(StructlogHandler())
        root_logger.setLevel(logging.NOTSET)
    else:
        root_logger = logging.getLogger()
        root_logger.handlers = [x for x in root_logger.handlers
                                if not isinstance(x, StructlogHandler)]
    # Configure structlog
    context_class = None
    if thread_local_context:
        context_class = structlog.threadlocal.wrap_dict(dict)
    structlog.reset_defaults()
    structlog.configure(
        processors=[
            fltr,
            add_level,
            add_pid,
            add_extra_context,
            structlog.processors.TimeStamper(fmt="iso", utc=True),
            add_exception_info,
            structlog.stdlib.PositionalArgumentsFormatter(),
            structlog.processors.UnicodeDecoder(),
            # See https://stackoverflow.com/a/51629142
            # we do the formatting in the Logger
            lambda _, __, ed: ed
        ],
        cache_logger_on_first_use=True,
        wrapper_class=MFBoundLogger,
        context_class=context_class,
        logger_factory=MFLogLoggerFactory()
    )
    CONFIGURATION_SET = True


def add_override(logger_name_pattern, minimal_level_name):
    """Add an override to the configuration.

    You provide a fnmatch pattern to the logger_name as the first argument.
    And the minimal_level_name (WARNING, DEBUG...) to force for this pattern.

    Note: if you use None as minimal_level_name, it will delete the override.

    """
    if not CONFIGURATION_SET:
        set_config()
    if minimal_level_name is None:
        try:
            # pylint: disable=unsupported-delete-operation
            del(Config.override_dict[logger_name_pattern])
        except KeyError:
            pass
    else:
        # just to raise an exception here
        # if the minimal_level_name is incorrect
        level_name_to_level_no(minimal_level_name)
        d = Config.override_dict
        # pylint: disable=unsupported-assignment-operation
        d[logger_name_pattern] = minimal_level_name
    __reset_level_from_logger_name_cache()


def getLogger(logger_name='root'):
    """Return a python logging logger.

    This function is just a wrapper.

    But by importing and using this one (and not directly logging.getLogger
    or structlog.get_logger),
    you are sure that the logging config is set.
    """
    if not CONFIGURATION_SET:
        set_config()
    return structlog.get_logger(logger_name, name=logger_name)


def get_logger(logger_name='root'):
    """Return a python logging logger.

    This function is just a wrapper.

    But by importing and using this one (and not directly logging.getLogger
    or structlog.get_logger),
    you are sure that the logging config is set.
    """
    return getLogger(logger_name)


def debug(message, *args, **kwargs):
    return get_logger().debug(message, *args, **kwargs)


def info(message, *args, **kwargs):
    return get_logger().info(message, *args, **kwargs)


def warning(message, *args, **kwargs):
    return get_logger().warning(message, *args, **kwargs)


def error(message, *args, **kwargs):
    return get_logger().error(message, *args, **kwargs)


def critical(message, *args, **kwargs):
    return get_logger().critical(message, *args, **kwargs)


def exception(message, *args, **kwargs):
    return get_logger().exception(message, *args, **kwargs)


def die(*args, **kwargs):
    get_logger().die(*args, **kwargs)


def dump_locals(f=sys.stderr):
    get_logger().dump_locals(f=f)


def __unset_configuration():
    global CONFIGURATION_SET
    CONFIGURATION_SET = False



import fastentrypoints  # noqa: F401
from setuptools import setup
from setuptools import find_packages

with open('requirements.txt') as reqs:
    install_requires = [
        line for line in reqs.read().split('\n')
        if (line and not line.startswith('--')) and (";" not in line)]

with open("README.md") as f:
    long_description = f.read()

#Version "0.0.0" will be replaced by CI when releasing
setup(
    author="Fabien MARTY",
    author_email="fabien.marty@gmail.com",
    name='mflog',
    version="v0.0.4",
    license="BSD 3",
    python_requires='>=2.7',
    url="https://github.com/metwork-framework/mflog",
    description="opinionated python (structured) logging library "
    "built on structlog",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=install_requires,
    entry_points={
        "console_scripts": [
            "log = mflog.log:main",
        ]
    }
)



#!/bin/env python3

import argparse
from mflog import get_logger


def main():
    parser = argparse.ArgumentParser("log a message with standard metwork "
                                     "logging system")
    parser.add_argument('--application-name', '-a', action="store",
                        default="default", help="application name")
    parser.add_argument('LEVEL', action='store',
                        help="Log level",
                        choices=['ERROR', 'CRITICAL', 'WARNING', 'INFO',
                                 'DEBUG'])
    parser.add_argument('MESSAGE', action='store', help="message to log")
    options = parser.parse_args()

    logger = get_logger(options.application_name)
    if options.LEVEL == 'DEBUG':
        logger.debug(options.MESSAGE)
    elif options.LEVEL == 'INFO':
        logger.info(options.MESSAGE)
    elif options.LEVEL == 'WARNING':
        logger.warning(options.MESSAGE)
    elif options.LEVEL == 'CRITICAL':
        logger.critical(options.MESSAGE)
    elif options.LEVEL == 'ERROR':
        logger.error(options.MESSAGE)
    else:
        raise Exception("Bad message level: %s", options.LEVEL)


if __name__ == "__main__":
    main()
